<?php
/*
Parmetros:
	action 		= ao requisitada
	token 		= Chave de temporria de operao acesso
	email		= Email de usurio cadastrado no projeto <pid>
	password	= Senha de acesso do usurio <email> no sistema
	pid			= Nmero identificador nico do projeto no sistema
	idform 		= Nmero identificador nico do formulrio no sistema
	idrecord 	= Nmero identificador do registro no sistema
*/
class Api {
	var $strEncoding = 'ISO-8859-1';
	var $intLimitResults = 100;
	var $intTokenValidMinutes = 30;
	var $intDailyRecordsCreationLimit = 100;
	var $strURLAPI;
	
	public function __construct($strPrefix = '') {
		include_once 'database.php';
		$database = new Database($strPrefix);
		$this->database = $database;
		include_once 'strings.php';
		$strings = new Strings();
		$this->strings = $strings;
		include_once 'project.php';
		$project = new Project($database);
		$this->project = $project;
		include_once 'user.php';
		$user = new User($database);
		$this->user = $user;
		include_once 'form.php';
		$form = new Form($database);
		$this->form = $form;
		include_once 'formrecord.php';
		$formrecord = new Formrecord($database);
		$this->formrecord = $formrecord;
		$this->strURLAPI = $this->strings->strSiteURL . '/api.php?';
	}
	
	public function strPrintErrorParametersInvalid($strParameters, $strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -1;
		$arrResult['detail'] = 'Incomplete parameters. Parameters required for this call: ' . $strParameters;
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorUserProjectInvalid($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -2;
		$arrResult['detail'] = 'Invalid email and/or password and/or Project ID';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorUserInvalid($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -2;
		$arrResult['detail'] = 'Invalid email and/or password';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorTokenInvalid($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -2;
		$arrResult['detail'] = 'Invalid token (expired after ' . $this->intTokenValidMinutes . ' minutes of its generation).';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}

	public function strPrintErrorNotPublicProject($strClass, $strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -3;
		$arrResult['detail'] = $strClass . ' cannot be listed without a token generated by an user from this project.  This project is not open for public access.  Thus, it is necessary to generate a token by command login';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorDeletePermission($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -4;
		$arrResult['detail'] = 'Record cannot be delete due to access level limitation.';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}

	public function strPrintErrorRecordNotExists($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -5;
		$arrResult['detail'] = 'Record does not exist!';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}

	public function strPrintErrorUserPositionInsertPermission($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -4;
		$arrResult['detail'] = 'User position could not be inserted due to access level limitation';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorRecordInsertExceeded($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -6;
		$arrResult['detail'] = 'Fail! Exceeded daily creation quota (' . $this->intDailyRecordsCreationLimit . ' records/day).';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintLoginTokenGenerated($strToken, $strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = $strToken;
		$arrResult['detail'] = 'Token successfully generated.  Valid for ' . $this->intTokenValidMinutes . ' minutes.';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintRecordDeleteOK($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrResult['detail'] = 'Record successfully deleted!';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strUserPositionInsertOK($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrResult['detail'] = 'User position successfully inserted!';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintReportGenerationOK($strOutputFile, $strOutputFormat) {
		$strExtension = explode('.', $strOutputFile);
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrResult['detail'] = 'Report successfully generated!';
		$arrResult['fileName'] = $this->strings->strSiteURL . '/' . $strOutputFile;
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorFormCreationNotPermited($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -4;
		$arrResult['detail'] = 'Record could not be created due to user level limitation. User access level must be >= 3.';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintErrorFormNotExists($strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = -5;
		$arrResult['detail'] = 'Form does not exist.';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	public function strPrintRecordInsertOK($idFormRecord, $strOutputFormat) {
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrResult['detail'] = 'Record successfully created!';
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result');
		}
	}
	
	// Verificar se o projeto idProjeto  pblico.  
	public function blnIsProjectPublic($idProject) {
		$arrProjects = $this->project->selectProjects($idProject);
		return $arrProjects[0]['blnAllowAllView'];
	}

	// Verificar se o token  vlido para X minutos 
	public function arrIsTokenValid($strToken) {
		$query = "
			SELECT 
				u.idUser,
				u.idProject,
				u.idUserLevel
			FROM
				userapisession api,
				user u
			WHERE 
				u.idUser	= api.idUser AND
				api.strKey	= '$strToken' AND
				(NOW() - INTERVAL " . $this->intTokenValidMinutes . " MINUTE) < api.dtDate;";
		$result = $this->database->executeQuery($query);
		$result = $this->database->prepareReturn($result);
		if (count($result) > 0) {
			return $result[0];
		} else {
			return false;
		}
	}
	
	public function blnIsUserInProject($idUser, $idProject){
		$query = "
		SELECT 
			COUNT(u.idProject)  AS `int`
		FROM 
			user u
		WHERE 
			u.idProject = " . $idProject . " AND 
			u.strEmail = (SELECT u.strEmail FROM user u WHERE u.idUser = " . $idUser . ")
		";
		$result = $this->database->executeQuery($query);
		$result = $this->database->prepareReturn($result);
		return $result[0]['int'] > 0;
	}

	// Verificar se a quantidade de registros criados hoje foi maior que $this->intDailyRecordsCreationLimit
	public function verificaUsuarioExtrapolouLimiteCriacaoDiaria($idUsuario) {
		$query = "SELECT COUNT(idFormRecord) AS as `int` FROM formrecord WHERE idUser=" . $idUsuario . " AND dtCreation >= DATE(NOW())";
		$result = $this->database->executeQuery($query);
		$result = $this->database->prepareReturn($result);
		return $result[0]['int'] >= $this->intDailyRecordsCreationLimit;
	}
	
	public function login($strEmail, $strPassword, $blnReturnArray = false, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ((! $strEmail) || (!$strPassword)) {
			$strResposta = $this->strPrintErrorParametersInvalid('action, email, password', $strOutputFormat);
			return $strResposta;
		}
		// Se continuou  porque as variveis esto OK
		$arrUsers = $this->user->mobileLogin($strEmail, md5($strPassword));
		// Se usurio existe vou preparar o retorno OK
		if (count($arrUsers) > 0) {
			$strToken = substr(md5(uniqid(time())),0,20);
			// Removendo registros de sessao anteriores.  S quero o ltimo
			$query = "DELETE FROM userapisession WHERE idUser=" . $arrUsers[0]['idUser'];
			$result = $this->database->executeQuery($query);
			// Inserindo registro dessa sessao
			$query = "INSERT INTO userapisession (idUser, dtDate, strKey) VALUES (" . $arrUsers[0]['idUser'] . ", NOW(), '" . $strToken . "')";
			$result = $this->database->executeQuery($query);
			if ($blnReturnArray) {
				$arrResult = array();
				$arrResult['strToken'] = $strToken;
				$arrResult['strEmail'] = $strEmail;
				$arrResult['strPassword'] = $strPassword;
				return $arrResult;
			} else {
				return $this->strPrintLoginTokenGenerated($strToken, $strOutputFormat);
			}
		} else {
			// Usurio no existe
			return $this->strPrintErrorUserProjectInvalid($strOutputFormat);
		}
	}
	
	public function listuserprojects($strToken, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ($strToken) {
			$arrToken = $this->arrIsTokenValid($strToken);
			if (! $arrToken) {
				return $this->strPrintErrorTokenInvalid($strOutputFormat);
			}
		} else {
			$strResposta = $this->strPrintErrorParametersInvalid('action, token', $strOutputFormat);
			return $strResposta;
		}
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrProjects = $this->user->selectUserProjects($arrToken['idUser']);
		$arrResult['projects'] = $arrProjects;
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result', null);
		}
	}	

	public function listprojectusers($idProject, $strToken, $strOutputFormat = 'xml') {
		if (! $idProject) return $this->strPrintErrorParametersInvalid('action, pid, token (for private projects)', $strOutputFormat);
		if (! $this->blnIsProjectPublic($idProject)) {
			if ($strToken) {
				$arrToken = $this->arrIsTokenValid($strToken);
				if (! $arrToken) {
					return $this->strPrintErrorTokenInvalid($strOutputFormat);
				}
				if (! $this->blnIsUserInProject($arrToken['idUser'], $idProject)) {
					return $this->strPrintErrorNotPublicProject('', $strOutputFormat);
				}
			} else {
				return $this->strPrintErrorNotPublicProject('Users', $strOutputFormat);
			}
		}
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrProjects = $this->project->selectProjects($idProject);
		$arrResult['project'] = $arrProjects[0];
		$arrUsers = $this->user->selectUsers($idProject);
		// Delete unnecessary field values
		foreach($arrUsers as $i => $arrUser){
			unset($arrUsers[$i]['strPassword']);
			unset($arrUsers[$i]['blnDeleted']);
		}
		$arrResult['users'] = $arrUsers;
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result', null);
		}
	}
	
	public function listprojectforms($idProject, $strToken, $strOutputFormat = 'xml') {
		if (! $idProject) return $this->strPrintErrorParametersInvalid('action, pid, token (for private projects)', $strOutputFormat);
		if (! $this->blnIsProjectPublic($idProject)) {
			if ($strToken) {
				$arrToken = $this->arrIsTokenValid($strToken);
				if (! $arrToken) {
					return $this->strPrintErrorTokenInvalid($strOutputFormat);
				}
				if (! $this->blnIsUserInProject($arrToken['idUser'], $idProject)) {
					return $this->strPrintErrorNotPublicProject('', $strOutputFormat);
				}
			} else {
				return $this->strPrintErrorNotPublicProject('Forms', $strOutputFormat);
			}
		}
		$arrResult = array();
		$arrResult['id'] = 1;
		$arrForms = $this->form->selectForms($idProject);
		$arrResult['forms'] = $arrForms;
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result', null);
		}
	}
	
	public function listforminfo($idForm, $strToken, $strOutputFormat = 'xml') {
		if (! $idForm) return $this->strPrintErrorParametersInvalid('idform, token (for private projects)', $strOutputFormat);
		$arrForms = $this->form->selectForms(0, $idForm);
		if (! count($arrForms)) return $this->strPrintErrorFormNotExists($strOutputFormat);
		$idProject = $arrForms[0]['idProject'];
		if (! $this->blnIsProjectPublic($idProject)) {
			if ($strToken) {
				$arrToken = $this->arrIsTokenValid($strToken);
				if (! $arrToken) {
					return $this->strPrintErrorTokenInvalid($strOutputFormat);
				}
				if (! $this->blnIsUserInProject($arrToken['idUser'], $idProject)) {
					return $this->strPrintErrorNotPublicProject('', $strOutputFormat);
				}
			} else {
				return $this->strPrintErrorNotPublicProject('Form', $strOutputFormat);
			}
		}
		$arrResult = array();
		$arrResult['form'] = $arrForms[0];
		$arrFormFields = $this->form->selectFormFields($arrForms[0]['idForm']);
		// Load alternatives for Unique or Multi Choice fields
		foreach ($arrFormFields as $i => $arrFormField) {
			if (($arrFormField['idFormFieldType'] == 3) || ($arrFormField['idFormFieldType'] == 4)) {
				$arrFormFieldAlternatives = $this->form->selectFieldAlternatives($arrFormField['idFormField']);
				$arrFormFields[$i]['alternatives'] = $arrFormFieldAlternatives;
			}
		}
		$arrResult['form']['fields'] = $arrFormFields;
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result', null);
		}
	}	
		
	public function deleterecord($idFormRecord, $strToken, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ((! $idFormRecord) || (! $strToken)) return $this->strPrintErrorParametersInvalid('idrecord, token', $strOutputFormat);
		$arrToken = $this->arrIsTokenValid($strToken);
		if (! $arrToken) return $this->strPrintErrorTokenInvalid($strOutputFormat); 
		// Verificar se o usurio tem permisso para remover o registro
		$arrFormRecord = $this->formrecord->selectFormRecords(0, 0, 0, $idFormRecord);
		// Registro no existe
		if (! count($arrFormRecord)) return $this->strPrintErrorRecordNotExists($strOutputFormat);
		$arrFormRecord = $arrFormRecord[0];
		// Somente nvel 4 e o proprietrio do registro pode remover
		if (($arrToken['idUserLevel'] < 4) && ($arrFormRecord['idUser'] != $arrToken['idUser'])) return $this->strPrintErrorDeletePermission($strOutputFormat);
		// Chamando funo para remover o registro
		$this->formrecord->deleteFormRecord($idFormRecord);
		return $this->strPrintRecordDeleteOK($strOutputFormat);
	}
	
	
	public function insertuserposition($dblLatitude, $dblLongitude, $strToken, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ((! $dblLatitude) || (! $dblLongitude) || (! $strToken)) return $this->strPrintErrorParametersInvalid('lat, lng, token', $strOutputFormat);
		$arrToken = $this->arrIsTokenValid($strToken);
		if (! $arrToken) return $this->strPrintErrorTokenInvalid($strOutputFormat); 
		if (($arrToken['idUserLevel'] < 4) && ($arrFormRecord['idUser'] != $arrToken['idUser'])) return $this->strPrintErrorUserPositionInsertPermission($strOutputFormat);
		// Chamando funo para inserir a posio do usurio
		$this->user->insertUserPosition($arrToken['idUser'], $dblLatitude, $dblLongitude);
		return $this->strUserPositionInsertOK($strOutputFormat);
	}
	
	public function insertrecord($idForm, $dblLatitude, $dblLongitude, $strData, $strToken, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ((! $idForm) || (! $strData) || (! $dblLatitude) || (! $dblLongitude) || (! $strToken)) return $this->strPrintErrorParametersInvalid('idform, lat, lng, data, token', $strOutputFormat);
		$arrToken = $this->arrIsTokenValid($strToken);
		if (! $arrToken) return $this->strPrintErrorTokenInvalid($strOutputFormat);
		if ($this->verificaUsuarioExtrapolouLimiteCriacaoDiaria($arrToken['idUser'])) return $this->strPrintErrorRecordInsertExceeded($strOutputFormat);		
		$arrForm = $this->form->selectForms(0, $idForm);
		if (! count($arrForm)) return $this->strPrintErrorFormNotExists($strOutputFormat);
		// Verificando se o usurio tem permisso para remover
		if ($arrToken['idUserLevel'] < 3) return $this->strPrintErrorFormCreationNotPermited($strOutputFormat);
		// Decupando dados para criar registro
		$arrData = explode('|', $strData);
	    $dblLatitude = str_replace(",", ".", $dblLatitude);
	    $dblLongitude = str_replace(",", ".", $dblLongitude);
		if (! is_numeric($dblLatitude)) $dblLatitude = 0;
		if (! is_numeric($dblLongitude)) $dblLongitude = 0;
		$arrCoordinates = array();
		$arrCoordinates[0]['dblLatitude'] = $dblLatitude;
		$arrCoordinates[0]['dblLongitude'] = $dblLongitude;
		$arrResult = $this->formrecord->insertFormRecord($arrToken['idUser'], $arrForm[0]['idForm'], 8, 1, '', $arrCoordinates);
		$idFormRecord = $arrResult['idFormRecord'];
		$arrFormFields = $this->form->selectFormFields($arrForm[0]['idForm']);
		$i=0;
		foreach ($arrFormFields as $arrFormField) {
			$strValor = $arrData[$i];
			$i++;
			if (! $strValor) continue;
			if ($arrFormField['idFormFieldType'] == 1) { 
				$this->formrecord->insertFormRecordFieldText($idFormRecord, $arrFormField['idFormField'], $strValue);
			} else if (($arrFormField['idFormFieldType'] == 2) && (is_numeric($strValor))){ 
				$this->formrecord->insertFormRecordFieldNumeric($idFormRecord, $arrFormField['idFormField'], $strValue);
			}
		}
		return $this->strPrintRecordInsertOK($idFormRecord, $strOutputFormat);
	}
		
	// $intSearchType = 1 - Search By Radius | = 2 - Search By Rect
	public function search ($intSearchType, $idProject, $dblLatitudeSW, $dblLongitudeSW, $dblLatitudeNE, $dblLongitudeNE, $intRadius, $strKeyword = '', $strToken, $intStart = 0, $idForm = 0, $dtDateFrom = '', $dtDateTo = '', $intLimit = 0, $strOutputFormat = 'xml') {
		set_time_limit(300);
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ($intSearchType == 1) { // Busca por raio
			if ((! $idProject) || (! is_numeric($dblLatitudeSW)) || (! is_numeric($dblLongitudeSW)) || (! $intRadius)) return $this->strPrintErrorParametersInvalid('pid, lat, lng, radius, token (for private projects)', $strOutputFormat);
		} else { // Busca por retngulo
			if ((! $idProject) || (! $dblLatitudeSW) || (! $dblLongitudeSW) || (! $dblLatitudeNE) || (! $dblLongitudeNE)) return $this->strPrintErrorParametersInvalid('pid, latSW, lngSW, latNE, lngNE, token (for private projects)', $strOutputFormat);
		}
		if (! $this->blnIsProjectPublic($idProject)) {
			if ($strToken) {
				$arrToken = $this->arrIsTokenValid($strToken);
				if (! $arrToken) {
					return $this->strPrintErrorTokenInvalid($strOutputFormat);
				}
				if (! $this->blnIsUserInProject($arrToken['idUser'], $idProject)) {
					return $this->strPrintErrorNotPublicProject('', $strOutputFormat);
				}
			} else {
				return $this->strPrintErrorNotPublicProject('Forms', $strOutputFormat);
			}
		}
		$arrCoordinates = array();
		if ($intSearchType == 1) {
			$arrCoordinates[0]['dblLat'] = $dblLatitudeSW;
			$arrCoordinates[0]['dblLng'] = $dblLongitudeSW;
			$arrFormRecords = $this->formrecord->selectFormRecordsByPolygon('C', $idProject, $arrCoordinates, '', $intRadius, $intLimit, 'fr.idForm, fr.idFormRecord', $idForm, $dtDateFrom, $dtDateTo);
		} else {
			$arrCoordinates[0]['dblLat'] = $dblLatitudeNE;
			$arrCoordinates[0]['dblLng'] = $dblLongitudeNE;
			$arrCoordinates[1]['dblLat'] = $dblLatitudeSW;
			$arrCoordinates[1]['dblLng'] = $dblLongitudeSW;
			$arrFormRecords = $this->formrecord->selectFormRecordsByPolygon('R', $idProject, $arrCoordinates, '', '', $intLimit, 'fr.idForm, fr.idFormRecord', $idForm, $dtDateFrom, $dtDateTo);
		}

		$k = 0;
		$arrResult = array();
		foreach ($arrFormRecords as $arrFormRecordReport) {
			$arrFormRecord = $this->formrecord->selectFormRecords(0, 0, 0, $arrFormRecordReport['idFormRecord']);
			$arrFormRecord = $arrFormRecord[0];
			$arrResult['records'][$k]['formName'] = $arrFormRecord['strFormName'];
			$arrResult['records'][$k]['recordID'] = $arrFormRecord['idFormRecord'];
			$arrResult['records'][$k]['shapeType'] = $arrFormRecord['idShapeType'];
			$arrResult['records'][$k]['creationDate'] = $arrFormRecord['strCreation'];
			$arrResult['records'][$k]['creationEmail'] = $arrFormRecord['strEmail'];
			$arrResult['records'][$k]['lastUpdateDate'] = $arrFormRecord['strLastUpdate'];
			$arrResult['records'][$k]['sourceDevice'] = $arrFormRecord['idDevice'];
			$arrResult['records'][$k]['shapeColor'] = $arrFormRecord['strCustomMarkerRGBColor'];
			$arrResult['records'][$k]['markerImage'] = $arrFormRecord['strMarkerFullURL'];
			if ($arrFormRecordReport['dblDistanceFromCenterMts'] > 0) {
				 $arrResult['records'][$k]['distanceMts'] = number_format($arrFormRecordReport['dblDistanceFromCenterMts'], 2, '.', '');
			}				
			$arrResult['records'][$k]['attachedFiles'] = count($arrFormRecord['arrFiles']);
			// Coordinates
			foreach($arrFormRecord['arrVertex'] as $index => $arrVerte) {
				$arrResult['records'][$k]['coordinates'][$index]['latitude'] = $arrVerte['dblLatitude'];
				$arrResult['records'][$k]['coordinates'][$index]['longitude'] = $arrVerte['dblLongitude'];
			}
		
			$arrFormRecordDatas = $this->formrecord->selectFormRecordData($arrFormRecordReport['idFormRecord']);
			foreach($arrFormRecordDatas as $index => $arrFormRecordData) {
			//$strValue = $this->formrecord->returnInfoCampoFormulario($arrFormRecordData, $arrFormField['idPergunta'], $arrFormField['idTipoPergunta']);	
				$strValue = $arrFormRecordData['strValue'];
				// Format for time series
				if ($arrFormRecordData['idFormFieldType'] == 7) $strValue = str_replace('<:>',", ", $strValue);
				$arrResult['records'][$k]['fields'][$index]['name'] = $arrFormRecordData['strName'];
				$arrResult['records'][$k]['fields'][$index]['description'] = $arrFormRecordData['strTip'];
				$arrResult['records'][$k]['fields'][$index]['dataType'] = $arrFormRecordData['idFormFieldType'];
				$arrResult['records'][$k]['fields'][$index]['value'] = $strValue;
			}
			
			// Attached Files
			foreach($arrFormRecord['arrFiles'] as $index => $arrFile) {
				$arrResult['records'][$k]['files'][$index]['name'] = $arrFile['strFileOriginalName'];
				$arrResult['records'][$k]['files'][$index]['description'] = $arrFile['strDescription'];
				$arrResult['records'][$k]['files'][$index]['sizeMb'] = number_format(@filesize($arrFile['strFileNameFullPath'])/1000000, 2, '.','');
				$arrResult['records'][$k]['files'][$index]['url'] = $arrFile['strFileNameFullURL'];
				$arrResult['records'][$k]['files'][$index]['urlThumb'] = $arrFile['strFileThumbURL'];
			}
			$k++;
		} // End of FormRecord
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result', null);
		}
		/*
		$xml = new SimpleXMLElement("<?xml version=\"1.0\"?><result></result>");
		$arrResult = $this->strings->encodeArray($arrResult);
		$this->strings->array_to_xml($arrResult,$xml);
		$strXML = $xml->asXML();
		return $strXML;
		*/
	}

	// Informaes de um registro
	public function recordinfo($idFormRecord, $strToken, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if (! $idFormRecord) return $this->strPrintErrorParametersInvalid('idrecord, token (for private projects)', $strOutputFormat);
		$arrFormRecord = $this->formrecord->selectFormRecords(0, 0, 0, $idFormRecord);
		// Registro no existe
		if (! count($arrFormRecord)) return $this->strPrintErrorRecordNotExists($strOutputFormat);
		if (! $this->blnIsProjectPublic($arrFormRecord[0]['idProject'])) {
			if ($strToken) {
				$arrToken = $this->arrIsTokenValid($strToken);
				if (! $arrToken) {
					return $this->strPrintErrorTokenInvalid($strOutputFormat);
				}
				if (! $this->blnIsUserInProject($arrToken['idUser'], $arrFormRecord[0]['idProject'])) {
					return $this->strPrintErrorNotPublicProject('', $strOutputFormat);
				}
			} else {
				return $this->strPrintErrorNotPublicProject('Forms', $strOutputFormat);
			}
		}
		if ($strOutputFormat == 'json') {
			return json_encode($this->strings->encodeArray($arrResult));
		} else {
			return $this->strings->strArrayToXML($arrResult, 'result', null);
		}
	}
			
	public function generatereport($idProject, $strExtension, $strToken, $strOutputFormat = 'xml') {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if ((! $idProject) || (! $strExtension)) return $this->strPrintErrorParametersInvalid('pid, ext, token (for private projects)', $strOutputFormat);
		if (! $this->blnIsProjectPublic($idProject)) {
			if ($strToken) {
				$arrToken = $this->arrIsTokenValid($strToken);
				if (! $arrToken) {
					return $this->strPrintErrorTokenInvalid($strOutputFormat);
				}
				if (! $this->blnIsUserInProject($arrToken['idUser'], $idProject)) {
					return $this->strPrintErrorNotPublicProject('', $strOutputFormat);
				}
				$arrProject = $this->project->selectProjects($idProject);
				$strJSONScope = json_encode($this->user->arrUserViewScope($arrToken['idUser'], $arrProject[0]['idProjectUserViewScope']));
			} else {
				return $this->strPrintErrorNotPublicProject('Forms', $strOutputFormat);
			}
		}
		include_once 'report.php';
		$report = new Report($this->database);
		$arrReturn = $report->generateReport($idProject, $strExtension, '', true, false, false, 0, 0, 70, '', '../', $strJSONScope);
		$strOutputFileName = $arrReturn['strFilePathTMP'];
		$strOutputFileName = str_replace("../", "", $strOutputFileName);
		return $this->strPrintReportGenerationOK($strOutputFileName, $strOutputFormat);
	}
	
	public function wfs($idProject, $strEmail = '', $strPassword = '', $blnLoadRecordInfo = false, $REQUEST, $TYPENAME) {
		// Verificando se as variveis esto OK.  Se no estiverem vou parar aqui
		if (!$idProject) {
			$strResposta = $this->strPrintErrorParametersInvalid('action, pid', $strOutputFormat);
			return $strResposta;
		}
		// Se projeto no  pblico, verificar se usurio est correto
		if (! $this->blnIsProjectPublic($idProject)) {
			$arrUsers = $this->user->mobileLogin($strEmail, md5($strPassword), $idProject);
			// Se usurio/senha incorreto, no aceitar
			if (! count($arrUsers)) {
				return $this->strPrintErrorUserProjectInvalid($strOutputFormat);
			}
		}
		
		if ($REQUEST == 'DescribeFeatureType') {
			return $this->wfsDescribeFeatureType();
		} else if (($REQUEST == 'GetFeature') && ($TYPENAME)) { // Redirect to Features Load
			return $this->wfsGetFeatures($TYPENAME, $blnLoadRecordInfo);
		} 
		//return file_get_contents("http://okmaps.org/geoserver/wfs?VERSION=1.1.0&REQUEST=GetCapabilities&SERVICE=WFS");
		// If continues, parameters OK		
		$strWFSURL = $this->strURLAPI . 'action=wfs&amp;pid=' . $idProject . '&amp;email=' . $strEmail . '&amp;password=' . $strPassword;
		$arrProject = $this->project->selectProjects($idProject);
		$arrProject = $arrProject[0];
		$xmlWFSCapabilities = new SimpleXMLElement('<WFS_Capabilities
		version="1.0.0"
		xmlns="http://www.opengis.net/wfs"
		xmlns:ogc="http://www.opengis.net/ogc"
		xmlns:gml="http://www.opengis.net/gml"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:ows="http://www.opengis.net/ows" 
		></WFS_Capabilities>');

		/*
		$xmlService = $xmlWFSCapabilities->addChild('Service');
		$xmlService->addChild('Name', 'WFS');
		$xmlService->addChild('Title', utf8_encode($arrProject['strName'] . ' (' . $this->strings->strSiteName . ')'));
		$xmlService->addChild('Abstract', $arrProject['strDescription'] ? utf8_encode($arrProject['strDescription']) : '');
		$xmlService->addChild('OnlineResource', $strWFSURL);
		$xmlService->addChild('Fees', 'NONE');
		$xmlService->addChild('SRS', 'EPSG');
		$xmlService->addChild('Fees', 'NONE');
		
		$xmlCapability = $xmlWFSCapabilities->addChild('Capability');
		$xmlCapability->addAttribute('xmlns:ogc', 'http://www.opengis.net/ogc');
		$xmlCapability->addAttribute('xmlns:gml', 'http://www/opengis.net/gml');
		$xmlRequest = $xmlCapability->addChild('Request');
		$xmlGetCapabilities = $xmlRequest->addChild('GetCapabilities');
		$xmlDCPType = $xmlGetCapabilities->addChild('DCPType');
		$xmlHTTP = $xmlDCPType->addChild('HTTP');
		$xmlGet = $xmlHTTP->addChild('Get');
		$xmlGet->addAttribute('onlineResource', str_replace('&amp;',"&", $strWFSURL) . '&REQUEST=GetCapabilities');
		
		$xmlDescribeFeatureType = $xmlRequest->addChild('DescribeFeatureType');
		$xmlSchemaDescriptionLanguage = $xmlDescribeFeatureType->addChild('SchemaDescriptionLanguage');
		$xmlDCPType = $xmlDescribeFeatureType->addChild('DCPType');
		$xmlHTTP = $xmlDCPType->addChild('HTTP');
		$xmlGet = $xmlHTTP->addChild('Get');
		$xmlGet->addAttribute('onlineResource', str_replace('&amp;',"&", $strWFSURL) . '&REQUEST=DescribeFeatureType');

		$xmlGetFeature = $xmlRequest->addChild('GetFeature');
		$xmlResultFormat = $xmlGetFeature->addChild('ResultFormat');
		$xmlGML2 = $xmlResultFormat->addChild('GML2');
		$xmlGML2 = $xmlResultFormat->addChild('GML3');
		$xmlXMLSCHEMA = $xmlSchemaDescriptionLanguage->addChild('XMLSCHEMA');
		$xmlDCPType = $xmlGetFeature->addChild('DCPType');
		$xmlHTTP = $xmlDCPType->addChild('HTTP');
		$xmlGet = $xmlHTTP->addChild('Get');
		//$xmlGet->addAttribute('onlineResource', 'http://copadesvr02.copade.neuquen.gov.ar/geoserver/wfs?request=GetFeature');
		$xmlGet->addAttribute('onlineResource', str_replace('&amp;',"&", $strWFSURL) . '&REQUEST=GetFeature');
		*/

		$arrForms = $this->form->selectForms($idProject);
		$xmlFeatureTypeList = $xmlWFSCapabilities->addChild('FeatureTypeList');
		foreach ($arrForms as $i => $arrForm) {
			$xmlFeatureType = $xmlFeatureTypeList->addChild('FeatureType');
			$xmlFeatureType->addChild('Title', utf8_encode($arrForm['strName']) . ' (' . $arrForm['intRecords'] . ')');
			$xmlFeatureType->addChild('Name', $arrForm['idForm']);
			$xmlFeatureType->addChild('SRS', 'EPSG:4326'); // WGS84
			
			/*
			$xmlLatLongBoundingBox = $xmlFeatureType->addChild('LatLongBoundingBox'); // WGS84
			$xmlLatLongBoundingBox->addAttribute('minx', '-180.0');
			$xmlLatLongBoundingBox->addAttribute('miny', '-90.0');
			$xmlLatLongBoundingBox->addAttribute('maxx', '180.0');
			$xmlLatLongBoundingBox->addAttribute('maxy', '90.0');
			*/
		}
		return $xmlWFSCapabilities->asXML();
	}
	
	public function wfsDescribeFeatureType() {
		//return file_get_contents("rascunho\WFS\Copa-DescribeFeatureType.xml");
		return '<xs:schema
			targetNamespace="http://www.supermap.com/World"
			xmlns:World="http://www.supermap.com/World" xmlns:gml="http://www.opengis.net/gml"
			xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified"
			attributeFormDefault="unqualified" version="1.0.0"
		></xs:schema>';
		
		$xmlSchema = new SimpleXMLElement('<schema></schema>');
        $xmlSchema->addAttribute('xmlns:ms', 'http://mapserver.gis.umn.edu/mapserver');
        $xmlSchema->addAttribute('xmlns:xsd', 'http://www.w3.org/2001/XMLSchema');
		$xmlSchema->addAttribute('xmlns:gml', 'http://www.opengis.net/gml');
		$xmlSchema->addAttribute('targetNamespace', 'http://mapserver.gis.umn.edu/mapserver');
		$xmlSchema->addAttribute('elementFormDefault', 'qualified');
		$xmlSchema->addAttribute('version', '0.1');
		$xmlImport = $xmlSchema->addChild('import');
		$xmlImport->addAttribute('namespace', 'http://www.opengis.net/gml');
		$xmlImport->addAttribute('schemaLocation', 'http://schemas.opengis.net/gml/2.1.2/feature.xsd');
		return $xmlSchema->asXML();
	}
	
	public function wfsGetFeatures($idForm, $blnLoadRecordInfo = false) {
		//return file_get_contents("rascunho\WFS\Copa-GetFeature.xml");
		$xmlFeatures = simplexml_load_string('<wfs:FeatureCollection
			xmlns:wfs="http://www.opengis.net/wfs"
			xmlns:gml="http://www.opengis.net/gml"
			xmlns:ogc="http://www.opengis.net/ogc"
			xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
			xsi:schemaLocation="http://www.opengis.net/wfs http://schemas.opengeospatial.net/wfs/1.0.0/WFS-basic.xsd"
		></wfs:FeatureCollection>');
		$gmlFeatureMembers = $xmlFeatures->addChild('gml:featureMembers', '', 'http://www.opengis.net/gml');
		// Form Records
//		$arrFormRecords = $this->formrecord->selectFormRecords(0, 0, $idForm);
$arrFormRecords = $this->formrecord->selectFormRecords(0, 0, $idForm, 0, '', 'idFormRecord', 10);
		foreach($arrFormRecords as $j => $arrFormRecord) {
			$gmlFeatureMember = $gmlFeatureMembers->addChild('gml:featureMember');
			// Print Record Coordinates
			if ($arrFormRecord['idShapeType'] == 1) {
				$gmlCoordinatesParent = $gmlFeatureMember->addChild('gml:Point');
			} else if ($arrFormRecord['idShapeType'] == 2) {
				$gmlCoordinatesParent = $gmlFeatureMember->addChild('gml:LineString');
			} else if ($arrFormRecord['idShapeType'] == 3) {
				$gmlPolygon = $gmlFeatureMember->addChild('gml:Polygon');
				$gmlPolygon->addAttribute('srsName', 'EPSG:4326');
				$gmlOuterBoundaryIs = $gmlPolygon->addChild('gml:outerBoundaryIs');
				$gmlCoordinatesParent = $gmlOuterBoundaryIs->addChild('gml:LinearRing');				
			}
			$gmlCoordinatesParent->addChild('gml:coordinates', $this->strings->strStringfyCoordinatesKML($arrFormRecord['arrVertex']));
			// Print Record Basic URL Info
			$gmlFeatureInfo = $gmlFeatureMember->addChild('FeatureInfo');
			//$gmlFeatureInfo->addChild('FormName', utf8_encode($arrForm['strName']));
			//$gmlFeatureInfo->addChild('URL', $this->strings->strSiteURL . '/' . $arrFormRecord['idFormRecord']);
			// Print Record Detailed
			if (! $blnLoadRecordInfo) continue;
			$arrFormRecordDatas = $this->formrecord->selectFormRecordData($arrFormRecord['idFormRecord']);
			foreach($arrFormRecordDatas as $k => $arrFormRecordData) {
				$strValue = $arrFormRecordData['strValue'];
				// Format for time series				
				if ($arrFormRecordData['idFormFieldType'] == 7) $strValue = str_replace('<:>',", ", $strValue);
				//$gmlFeatureInfo->addChild(utf8_encode($arrFormRecordData['strName']), utf8_encode($strValue));
			}
		}
		/*
		$gmlBoundedBy = $xmlFeatures->addChild('gml:boundedBy','', 'http://www.opengis.net/gml');
		$gmlBox = $gmlBoundedBy->addChild('gml:Box');
		$gmlBox->addAttribute('srsName', 'EPSG:4326');
		$gmlCoordinates = $gmlBox->addChild('gml:coordinates', '-180.000000,-90.000000 180.000000,83.627419');
		*/
		return $xmlFeatures->asXML();
	}
}
?>